import {ChartViewService} from "@/Services/ChartViewService";
import {Utilities} from "@/Services/Utilities";
import { ZmartChartsApex } from "@/Services/Chats/ZmartChartsApex";
import { FilterModalController } from "@/controllers/FilterModalController";
import { Filter } from '@/Services/Filter/Filter';

export class ChartViewAbstract {
    protected factMaskFunction: any;
    protected dimensionMaskFunction: any;
    protected zmartCharts: any;
    protected viewData: any;
    public period: any = {};

    public hasMultipleDimensions: Boolean = false;
    public hasMultipleFacts: Boolean = true;

    public drillDownFilters: any = {};

    public getSaveViewParams(formExtra: any, values: any, cleanValues: any) {
        const chartSeries = [
            {
                serieColor: [null],
                serieColors: values.serieColors || null,
                serieFields: values.serieFields || null,
                serieLabels: values.serieLabels || null,
                serieLineWidth: values.serieLineWidth || null,
                serieStacked: values.serieStacked || null,
                serieLineType: values.serieLineType || null,
                serieYAxisFormat: values.serieYAxisFormat || null,
                serieType: values.serieType || null
            }

        ];
        cleanValues.chartSeries = chartSeries;
        delete cleanValues.serieColors;
        return cleanValues;
    }
    getWidget(dashboard: any, widget: any) {
        const height = widget[0].style;
        if (dashboard.label !== '' || dashboard.label !== null) {
            widget[0].options.title.text = dashboard.label;
        }
        widget[0].style = "background: " + dashboard.color + "; " + height;
        return widget;
    }

    getWidgetDashboard(widget: any, dashboard: any) {
        return widget;
    }

    getZmartCharts() :ZmartChartsApex {
        if (!this.zmartCharts) {
            this.zmartCharts = new ZmartChartsApex();
        }
        return <ZmartChartsApex> this.zmartCharts;
    }

    public setDrill(charts :any, zmartCharts: ZmartChartsApex) {
        this.zmartCharts = zmartCharts;
        return charts;
    }

    getEditData(viewData: any) {
        return viewData;
    }
    setEditData(viewData: any) {
        this.viewData = viewData;
    }
    getCharts(charts: any) {
        return charts;
    }

    public async getFilterModalParameters() {
        const chart = this.zmartCharts.chartMetadata[0];
        let period = null;
        if (this.period) {
            if (Array.isArray(this.period) && this.period[0]) {
                period = [this.period[0].start + " até " + this.period[0].end];
            } else {
                period = [this.period.start + " até " + this.period.end];
            }
        }
        const structures = await Filter.filterGetStructureByUser();
        return {
            dimensions: this.getAllDimensions(chart.widget.columns),
            selectedDimension: chart.widget[chart.widget.dimensionProperty],
            facts: this.getAllFacts(chart.widget.columns),
            selectedFacts: this.getSelectedFacts(chart.widget.factProperties, chart.widget),
            hasMultipleDimensions: this.hasMultipleDimensions,
            hasMultipleFacts: this.hasMultipleFacts,
            drillDownFilters: this.drillDownFilters,
            structures: structures.data.dataset.getStructureByUser || [],
            selectedStructure: chart.widget.parameters.structureId[0] || undefined,
            period
        };
    }

    public getSelectedFacts(factProperties: any, widget: any) {
        if (!Array.isArray(factProperties)) {
            return widget[factProperties];
        }
        if (factProperties.length && factProperties.length === 1) {
            return widget[factProperties[0]];
        }
        const ret: any = [];
        factProperties.forEach((fact: any) => {
            ret.push(widget[fact]);
        });
        return ret.flat();
    }

    public getAllFacts(columns: any) {
        const facts = columns.filter((column: any) => column.isFact);
        const ret: any = [];
        facts.forEach((fact: any) => {
            ret.push({
                "id": fact.name,
                "name": fact.label,
            });
        });
        return ret;
    }

    public getAllDimensions(columns: any) {
        const dimensions = columns.filter((column: any) => !column.isFact);
        const ret: any = [];
        dimensions.forEach((dimension: any) => {
            ret.push({
                "id": dimension.name,
                "name": dimension.label,
            });
        });
        return ret;
    }

    public async getFilterModal() {
        const filterModalParameters = await this.getFilterModalParameters();
        const filterModal = new FilterModalController();
        return filterModal.getChartViewFilterModalJson(filterModalParameters);
    }
}
